/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2001 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: David Eriksson <eriksson@php.net>                            |
   +----------------------------------------------------------------------+
 */

#include "fromcorba.h"
#include "zppvalue.h"
#include "object.h"
#include "struct.h"
#include <iostream>
using namespace std;

void from_corba(CORBA::NamedValue_ptr pSource, ZppValue& rResult)
{
	from_corba(*pSource->value(), rResult);
}

void from_corba_sequence(CORBA::Any& rSource, ZppValue& rDestination, CORBA::TypeCode_ptr pType)
{
	CORBA::TypeCode_ptr content_type = pType->content_type();

	rDestination.Array();
	
	CORBA::ULong size = 0;
	rSource.seq_get_begin(size);

	for (unsigned i = 0; i < size; i++)
	{
		ZppValue element;
		from_corba(rSource, element, content_type);

		// save element in hashtable
		zval * p_element = element.Release();
		rDestination.HashTable().IndexAdd(&p_element);
	}

	rSource.seq_get_end();
	
}

void from_corba(CORBA::Any& rSource, ZppValue& rResult, CORBA::TypeCode_ptr pType)
{
	if (NULL == pType)
		pType = rSource.type(); 

	switch (pType->kind())
	{
		case CORBA::tk_short:
			{
				CORBA::Short value;
				if (rSource >>= value)
					rResult.Long(value);
			}
			break;
		case CORBA::tk_ushort:	
			{
				CORBA::UShort value;
				if (rSource >>= value)
					rResult.Long(value);
			}
			break;

		case CORBA::tk_long: 		
			{
				CORBA::Long value;
				if (rSource >>= value)
					rResult.Long( value ); 
			}
			break;

		case CORBA::tk_ulong:		
			{
				CORBA::ULong value;
				if (rSource >>= value)
					rResult.Long(value);
			}
			break;
													
		case CORBA::tk_float: 
			{
				CORBA::Float value;
				if (rSource >>= value)
					rResult.Double(value);
			}
			break;

		case CORBA::tk_double: 	
			{
				CORBA::Double value;
				if (rSource >>= value)
					rResult.Double(value);
			}
			break;

		case CORBA::tk_boolean:
			{
				CORBA::Boolean value;
				if (rSource >>= CORBA::Any::to_boolean(value))
					rResult.Bool( value != 0 );
			}
			break;
														
		case CORBA::tk_char:
			{
				CORBA::Char value;
				if (rSource >>= CORBA::Any::to_char(value))
					rResult.Long( value ); 
			}
			break;

		case CORBA::tk_octet: 
			{
				CORBA::Octet value;
				if (rSource >>= CORBA::Any::to_octet(value))
					rResult.Long(value);
			}
			break;
		
		case CORBA::tk_objref: 
			UniverseObject::ConvertFromCorba(rSource, rResult, pType);
			break;
		
		case CORBA::tk_struct:
		case CORBA::tk_except: 
			UniverseStruct::ConvertFromCorba(rSource, rResult, pType);
			break;

		case CORBA::tk_string: 
			{
				// who owns the memory pointed to by value? not me, I suppose
				const char * value;
				if (rSource >>= CORBA::Any::to_string(value, /* bound = */ 0))
					rResult.String(value);
			}
			break;
		
		case CORBA::tk_enum:
			{
				CORBA::ULong value;
				if (rSource.enum_get(value))
				{
					rResult.String( pType->member_name(value) );
//					rResult.Long(value);
				}
			}
			break;

		case CORBA::tk_alias: 
			// Use very nice MICO extension: TypeCode::unalias()
			from_corba(rSource, rResult, pType->unalias());
			break;
			
		case CORBA::tk_sequence: 
			from_corba_sequence(rSource, rResult, pType);
			break;
	
		case CORBA::tk_any: 
		case CORBA::tk_TypeCode: 
		case CORBA::tk_Principal: 
		case CORBA::tk_union: 
		case CORBA::tk_array: 
		case CORBA::tk_longlong: 
		case CORBA::tk_ulonglong: 
		case CORBA::tk_longdouble: 
		case CORBA::tk_wchar: 
		case CORBA::tk_wstring: 
		case CORBA::tk_fixed: 
		case CORBA::tk_value: 
		case CORBA::tk_value_box: 
		case CORBA::tk_native: 
		case CORBA::tk_abstract_interface: 
#if MICO_BIN_VERSION >= 0x020306
		case CORBA::tk_local_interface:
#endif
			// XXX: error unhandled type
			cerr << "Unhandled type: " << pType->kind() << endl;
			// fall through for now
	
		case CORBA::tk_null:
			// fall through
		case CORBA::tk_void: 
			rResult.Null();
			break;
	}
}

